package gs.maintainer.model

import cats.Eq
import cats.Show
import java.time.Instant
import scala.util.Try

/**
  * Represents the instant a [[Project]] was registered.
  */
opaque type RegisteredAt = Instant

/**
  * Represents the instant a [[Project]] was registered.
  */
object RegisteredAt:

  /**
   * Instantiate a new [[RegisteredAt]] from the given timestamp.
   *
   * @param value The timestamp.
   * @return The new [[RegisteredAt]] instance.
   */
  def apply(value: Instant): RegisteredAt = value

  /**
   * Parse the given string as a [[RegisteredAt]]. This function uses the
   * default parser for the `java.time.Instant` type (`ISO_INSTANT`).
   *
   * @param candidate The candidate string.
   * @return The new [[RegisteredAt]] instance, or an error if parsing failed.
   */
  def parse(candidate: String): Either[ModelError, RegisteredAt] =
    Try(Instant.parse(candidate))
      .toOption
      .map(Right(_))
      .getOrElse(Left(ModelError.InvalidTimestamp(candidate)))

  given CanEqual[RegisteredAt, RegisteredAt] = CanEqual.derived

  given Eq[RegisteredAt] = (x, y) => x == y

  given Show[RegisteredAt] = _.unwrap().toString()

  extension (ra: RegisteredAt)
    /**
     * @return The underlying `java.time.Instant`.
     */
    def unwrap(): Instant = ra

    /**
     * @return The epoch milliseconds represented by this timestamp.
     */
    def toEpochMillis(): Long = ra.toEpochMilli()

end RegisteredAt
